export("AUC", "Bahr", "BallDivergence", "BF", "BG", "BG2", "BMG", "BQS", "C2ST", 
       "CCS", "CCS_cat", "CF", "CF_cat", "CMDistance", 
       "Cramer", "DataSimilarity", "DiProPerm", "DISCOB", "DISCOF", "DS",
       "dwdProj", "Energy", "engineerMetric",
       "f.a", "f.s", "f.aCat", "f.sCat", "findSimilarityMethod", "findSigma", 
       "FR", "FStest", "FR_cat", "GGRL", "GGRLCat", "GPK", "gTests", 
       "gTests_cat", "gTestsMulti", "HamiltonPath", "hammingDist", "HMN", 
       "Jeffreys", "kerTests",
       "KMD", "knn", "knn.bf", "knn.fast", "LHZ", "LHZStatistic", "MD", 
       "MMCM", "MMD", "MST", "MST5", "MW", "NKT", "OTDD", "Petrie",
       "rectPartition", "RISE", "RItest", "Rosenbaum", "SC", "SH", "svmProj", "tStat", 
       "Wasserstein", "YMRZL", "ZC", "ZC_cat")

importFrom("stats", "dist", "pchisq", "pnorm", "var", "cov", "t.test", "na.omit")
importFrom("boot", "boot")
importFrom("utils", "combn")